//辅助函数小集合(js)

// 时间戳转换成日期格式 
// 参数1: 时间戳(秒或者毫秒) $time Number
// 参数2: 日期样式 $style Number 默认为1（分隔符形式） 1:分隔符  2:中文汉字
// 参数3: 分隔符 $division String 默认为 -字符 分割
// 返回值: 日期格式字符串 dateformat String
const changeDateDateformat = ($time, $style = 1, $division = '-') => {
	// 判断时间戳是否为13位数，如果不是则*1000，时间戳只有13位数(带毫秒)和10(不带毫秒)位数的
	let times = null;
	let dateformat = null;
	if ($time.toString().length == 13) {
		times = new Date($time)
	} else {
		times = new Date($time * 1000);
	}
	let Y = times.getFullYear(); //年
	let M = (times.getMonth() + 1 < 10 ? '0' + (times.getMonth() + 1) : times.getMonth() + 1); //月
	let D = times.getDate(); //日
	// let h = times.getHours(); //时
	// let m = times.getMinutes(); //分
	// let s = times.getSeconds(); //秒	
	// const dateformat = Y+$division+M+$division+D+$division+h+$division+m+$division+s;
	if ($style == 1) {
		dateformat = Y + $division + M + $division + D;
	} else if ($style == 2) {
		dateformat = M + $division + D;
	} else {
		dateformat = Y + '年 ' + M + '月 ' + D + '日 ';
	}
	return dateformat;
}

// 日期字符串转时间戳 
// 参数1: 时间格式字符串 $time String 示例："2016/01/01 17:22:37"
// 返回值: 时间戳(秒) timestamp Number
const changeDateTimestamp = ($time) => {
	console.log($time)
	// Date.parse() 输出从 1970/01/01 到一个 *具体日期* 的毫秒数； 时间可以自定义
	// getTime() 输出从 1970/01/01 到一个 *现在（new Date()）* 的毫秒数； 该方法总是结合一个 Date 对象来使用
	//new Date(date).getTime()在苹果手机不兼容
	//解决办法：new Date(data.replace(/-/g,'/')).getTime()
	let timestamp = (new Date($time.replace(/-/g, '/'))).getTime() / 1000;
	return timestamp
}

// 小型日历
//参数一： 请求结果回调 $cb Function 必填
const minCalendar = ($cb) => {
	//获取当前年月日
	let times = new Date();
	let Y = times.getFullYear(); //年
	let M = times.getMonth(); //月 0-11
	let D = times.getDate(); //日
	let W = times.getDay(); //今天星期几 0(星期天)-6(星期六)
	let h = times.getHours(); //时
	let m = times.getMinutes(); //分
	let timestamp = Date.parse(times); //当前的时间戳 毫秒 要*0.001
	console.log(Y + '年', M + '月(+1)', D + '日', '星期' + W + '——0(星期天)-6(星期六)', h + '小时', m + '分')

	let currentMonth = getMonthCount(Y, M) //当月天数
	let nextMonth = getMonthCount(Y, M + 1) //下月天数
	// let todayTime = Y+"-"+((M+1)<10?'0'+(M+1):(M+1))+"-"+(D<10?'0'+D:D)+" "+(h<10?'0'+h:h)+":"+(m<10?'0'+m:m)+":"+"00"//当前日期时间信息字符串
	let calendar = {
		'todayMonth': M + 1,
		'todayYear': Y,
		'today': D,
		'todayWeek': W,
		'timestamp': timestamp / 1000,
		'currentMonth': currentMonth,
		'nextMonth': nextMonth
	}
	if ($cb) {
		$cb(calendar)
	}
}
//小型日历-判断该年份是不是闰年 平年闰年[四年一闰，百年不闰，四百年再闰] 公历平年全年有365天，而公历闰年全年有366天
const isLeapYear = (year) => {
	return (year % 400 === 0) || (year % 100 !== 0 && year % 4 === 0);
}

//小型日历-获取该月份的天数
const getMonthCount = (year, month) => {
	let arr = [31, null, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]; //1-12月的天数
	let days = arr[month] || (isLeapYear(year) ? 29 : 28)
	return days;
}

export function cloneDeep(data) {
  //string,number,bool,null,undefined,symbol
  //object,array,date
  if (data && typeof data === "object") {
    //针对函数的拷贝
    if (typeof data === "function") {
      let tempFunc = data.bind(null);
      tempFunc.prototype = cloneDeep(data.prototype);
      return tempFunc;
    }
    // 根据不同的数据类型的深拷贝
    switch (Object.prototype.toString.call(data)) {
      case "[object String]":
        return data.toString();
      case "[object Number]":
        return Number(data.toString());
      case "[object Boolean]":
        return new Boolean(data.toString());
      case "[object Date]":
        return new Date(data.getTime());
      case "[object Array]":
        var arr = [];
        for (let i = 0; i < data.length; i++) {
          arr[i] = cloneDeep(data[i]);
        }
        return arr;

      //js自带对象或用户自定义类实例
      case "[object Object]":
        var obj = {};
        for (let key in data) {
          //会遍历原型链上的属性方法，可以用hasOwnProperty来控制 （obj.hasOwnProperty(prop)
          obj[key] = cloneDeep(data[key]);
        }
        return obj;
    }
  } else {
    //string,number,bool,null,undefined,symbol
    return data;
  }
}


export default {
	changeDateDateformat, //时间戳转换成日期
	changeDateTimestamp, //日期字符串转时间戳
	minCalendar, //小型日历
	cloneDeep
}
